//
//  NisusSpatialEdge.h
//  NisusCore
//
//  Created by Martin on 2019.01.28.
//  Copyright © 2019 Nisus. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <NisusCore/NisusUtil.h>

typedef struct {
	CGFloat location;
	CGFloat length;
} NisusSpatialEdge;

#define NisusSpatialEdge_Zero (NisusSpatialEdge_Make(0,0))

static inline NisusSpatialEdge NisusSpatialEdge_Make( CGFloat location, CGFloat length ) {
	NisusSpatialEdge edge = {location, length};
	return edge;
}
	
static inline CGFloat NisusSpatialEdge_Min( NisusSpatialEdge edge ) {
	return edge.location;
}
	
static inline CGFloat NisusSpatialEdge_Max( NisusSpatialEdge edge ) {
	return edge.location + edge.length;
}

static inline BOOL NisusSpatialEdge_IsEqual( NisusSpatialEdge edge1, NisusSpatialEdge edge2 ) {
	return (edge1.location == edge2.location) && (edge1.length == edge2.length);
}
	
static inline BOOL NisusSpatialEdge_IsEqualWithTolerance( NisusSpatialEdge edge1, NisusSpatialEdge edge2, CGFloat tolerance ) {
	return (ABS(edge1.location - edge2.location) <= tolerance) && (ABS(edge1.length - edge2.length) <= tolerance);
}

static inline NSComparisonResult NisusSpatialEdge_Compare( NisusSpatialEdge edge1, NisusSpatialEdge edge2 )
{
	NSComparisonResult res = NisusCompareScalars( edge1.location, edge2.location );
	if( NSOrderedSame == res ) {
		res = NisusCompareScalars( edge1.length, edge2.length );
	}
	return res;
}

static NisusSpatialEdge NisusSpatialEdge_MakeWithLocations( CGFloat loc1, CGFloat loc2 )
{
	if( loc1 < loc2 ) return NisusSpatialEdge_Make(loc1, loc2 - loc1);
	else if( loc1 > loc2 ) return NisusSpatialEdge_Make(loc2, loc1 - loc2);
	else return NisusSpatialEdge_Make(loc1, 0);
}

// zero length edges are ignored.
static NisusSpatialEdge NisusSpatialEdge_Union( NisusSpatialEdge edge1, NisusSpatialEdge edge2 )
{
	if( edge1.length <= 0 ) return edge2;
	if( edge2.length <= 0 ) return edge1;
	
	CGFloat minLoc = MIN( edge1.location, edge2.location );
	CGFloat maxLoc = MAX( edge1.location + edge1.length, edge2.location + edge2.length );
	return NisusSpatialEdge_Make(minLoc, maxLoc - minLoc);
}

// Checks if the two edges overlap at all.
// Zero-length edges are considered to be points and are checked.
static BOOL NisusSpatialEdge_IsOverlap( NisusSpatialEdge edge1, NisusSpatialEdge edge2 )
{
	if( edge1.location <= edge2.location ) {
		return (NisusSpatialEdge_Max(edge1) >= NisusSpatialEdge_Min(edge2));
	}
	else { // edge2.location <= edge1.location
		return (NisusSpatialEdge_Max(edge2) >= NisusSpatialEdge_Min(edge1));
	}
}



// conversions
extern NSString* NisusSpatialEdge_ToString( NisusSpatialEdge edge );

@interface NSValue (NisusSpatialEdge)

+ (instancetype) valueWithNisusSpatialEdge:(NisusSpatialEdge)edge ;
- (NisusSpatialEdge) nisusSpatialEdgeValue ;

@end





